//----------------------------------------------------------
//
//	low level device driver for LC7981 graphic LC displays
//
//----------------------------------------------------------



; commands
.equ LCD_MODE		= 0x00	;Mode Control
.equ LCD_CP			= 0x01	;Character Pitch
.equ LCD_NC			= 0x02	;Number of Characters
.equ LCD_TDN		= 0x03	;Time Division Number
.equ LCD_CURSP		= 0x04	;Curor Position
.equ LCD_DISPSL		= 0x08	;Display Start Lower Address	
.equ LCD_DISPSH		= 0x09	;Display Start Upper Address
.equ LCD_CURSAL		= 0x0A	;Cursor Lower Address
.equ LCD_CURSAH		= 0x0B	;Cursor Upper Address
.equ LCD_WRITED		= 0x0C	;Write Data
.equ LCD_READD		= 0x0D	;Read Data
.equ LCD_CLRBIT		= 0x0E	;Bit clear
.equ LCD_SETBIT		= 0x0F	;Bit set

; configuration attributes (mode register bits 6 and 7 remain cleared)
.equ ATTR_DISPLAY_ON  = 1 << 5
.equ ATTR_DISPLAY_OFF = 0 << 5
.equ ATTR_IS_MASTER   = 1 << 4
.equ ATTR_IS_SLAVE    = 0 << 4
.equ ATTR_CURSOR_OFF  = 0b0000
.equ ATTR_CURSOR_ON   = 0b0100 	; (text mode only)
.equ ATTR_CURSOR_OFF_CHAR_BLINK  = 0b1000 ; (text mode only)
.equ ATTR_CURSOR_BLINK = 0b1100 ; (text mode only)
.equ ATTR_GRAPHICMODE = 1 << 1
.equ ATTR_TEXTMODE 	  = 0 << 1
.equ ATTR_CHARGEN_EXT = 1 << 0 	; (text mode only)
.equ ATTR_CHARGEN_INT = 0 << 0 

; other constants
.equ LCD_BUSY_BIT	= 7
.equ ALL_INPUT		= 0b00000000 ; Port direction register bits
.equ ALL_OUTPUT		= 0b11111111 ; Port direction register bits






	LCD16080_ToggleENA: ; refer to datasheet
			rcall WAIT_10us
			sbi LCD16080_BITS, LCD16080_ENA 
			rcall WAIT_10us
			sbi LCD16080_BITS, LCD16080_ENA
			rcall WAIT_10us
		ret



	LCD16080_Wait:								; wait for display controller
			SetPortIOStat LCD16080_IOPORT, ALL_INPUT ; data direction : input
			rcall WAIT_10us
			sbi	LCD16080_BITS,LCD16080_RS		; RS=1 (change to command register)
			sbi	LCD16080_BITS,LCD16080_RW		; RW=1 (set display to write)
			rcall WAIT_10us
			sbi LCD16080_BITS, LCD16080_ENA
			rcall WAIT_10us	
				sbic	LCD16080_IOPORT-2 ,LCD_BUSY_BIT ; repeat "LCD_BUSY_BIT clear?"
				rjmp	pc - 1					; until LCD_BUSY clear
			rcall WAIT_10us
			cbi	LCD16080_BITS, LCD16080_ENA
			rcall WAIT_10us
		ret



	_LCD16080_Write:
		BEGINPROC

			pop XL ; Operator (instruction) << Stack 
			pop XH ; Operand  (data)        << Stack
						
			rcall LCD16080_Wait 				; make sure the line is clear
			
			sbi	LCD16080_BITS, LCD16080_RS		; RS=1 (change to command register)
			cbi	LCD16080_BITS, LCD16080_RW		; RW=0 (set display to read)
				
			SetPortIOStat LCD16080_IOPORT, ALL_OUTPUT ; data direction : output
				
			out	LCD16080_IOPORT, XL				; write instruction
			rcall LCD16080_ToggleENA

			cbi	LCD16080_BITS, LCD16080_RS		; RS=0 (change to data register)
			rcall WAIT_10us
			
			out	LCD16080_IOPORT, XH				; write data
			rcall LCD16080_ToggleENA
			
			rcall LCD16080_Wait					; finish

		ENDPROC



	LCD16080_Init:
			cbi LCD16080_BITS, LCD16080_RST
			sbi LCD16080_BITS, LCD16080_CS		   ;
			cbi LCD16080_BITS, LCD16080_ENA		   ; enable low
			cbi LCD16080_BITS, LCD16080_RW		   ; set display to read
			rcall WAIT_100ms
			sbi LCD16080_BITS, LCD16080_RST
			rcall WAIT_100ms
			cbi LCD16080_BITS, LCD16080_CS
		ret



	LCD16080_GraphicMode:
			LCD16080_Write LCD_MODE, ATTR_DISPLAY_ON | ATTR_IS_MASTER | ATTR_GRAPHICMODE
			LCD16080_Write LCD_CP,  0x07	; Pitch X = 8 pixels (at 1 bit / px)
			LCD16080_Write LCD_NC,  0x1F 	; 32 chars / line, lines 0..31
			LCD16080_Write LCD_TDN, 0x4F    ; time division number 79+1 : 1/80 timing
			LCD16080_Write LCD_DISPSL, 0x00 ; set start address 0x0000
			LCD16080_Write LCD_DISPSH, 0x00	; 
		ret



	LCD16080_TextMode:
			LCD16080_Write LCD_MODE, ATTR_DISPLAY_ON | ATTR_IS_MASTER | ATTR_TEXTMODE | ATTR_CHARGEN_INT | ATTR_CURSOR_OFF
			LCD16080_Write LCD_CP,  (8-1) << 4 | (6-1) << 0 ; char pitch Y = 8, char pitch X = 6
			LCD16080_Write LCD_NC,  0x18 	; 26 chars / line
			LCD16080_Write LCD_TDN, 0x4F    ; time division number 79+1 : 1/80 timing
			LCD16080_Write LCD_CURSAL, 0x00 ; set start at character 0, line 0
			LCD16080_Write LCD_CURSAH, 0x00	; 
		ret

